from __future__ import annotations

from . import generate_ca_bundle

if __name__ == "__main__":
    bundle = generate_ca_bundle()

    if not bundle:
        import platform

        print("System is not supported")
        print("system: ", platform.system())
        print("uname: ", platform.uname())
        print("python: ", platform.python_version_tuple())
        print("implementation: ", platform.python_implementation())

        exit(1)

    print(bundle)



from __future__ import annotations

from socket import AF_INET, SOCK_STREAM, socket
from ssl import SSLError

import pytest

from wassima import create_default_ssl_context


@pytest.mark.parametrize(
    "host, port, expect_failure",
    [
        ("1.1.1.1", 443, False),
        ("google.com", 443, False),
        (
            "self-signed.badssl.com",
            443,
            True,
        ),
        (
            "untrusted-root.badssl.com",
            443,
            True,
        ),
        (
            "tls-v1-2.badssl.com",
            1012,
            False,
        ),
        (
            "sha1-intermediate.badssl.com",
            443,
            True,
        ),
        ("one.one.one.one", 443, False),
        ("edellroot.badssl.com", 443, True),
        ("developer.mozilla.org", 443, False),
        ("letsencrypt.org", 443, False),
    ],
)
def test_ctx_use_system_store(host: str, port: int, expect_failure: bool) -> None:
    ctx = create_default_ssl_context()

    s = socket(AF_INET, SOCK_STREAM)
    s = ctx.wrap_socket(s, server_hostname=host)

    if expect_failure:
        with pytest.raises(SSLError) as exc:
            s.connect((host, port))
        ssl_err = exc.value.args[1]
        assert (
            "self-signed" in ssl_err
            or "self signed" in ssl_err
            or "unable to get local issuer certificate" in ssl_err
        )
    else:
        s.connect((host, port))
        assert s.getpeercert() is not None

    s.close()



"""
the Wassima library is a simple wrapper around the crate rustls-native-certs.
It aims to provide a pythonic way to retrieve root CAs from your system without any difficulties or hazmat.
"""
from __future__ import annotations

import ssl
from functools import lru_cache
from ssl import DER_cert_to_PEM_cert

from ._rustls import root_der_certificates


@lru_cache()
def root_pem_certificates() -> list[str]:
    """
    Retrieve a list of root certificate from your operating system trust store.
    They will be PEM encoded.
    """
    pem_certs = []

    for bin_cert in root_der_certificates():
        pem_certs.append(DER_cert_to_PEM_cert(bin_cert))

    return pem_certs


def generate_ca_bundle() -> str:
    """
    Generate an aggregated CA bundle that originate from your system trust store.
    Simply put, concatenated root PEM certificate.
    """
    return "\n\n".join(root_pem_certificates())


def create_default_ssl_context() -> ssl.SSLContext:
    """
    Instantiate a native SSLContext (client purposes) that ships with your system root CAs.
    In addition to that, assign it the default OpenSSL ciphers suite and set
    TLS 1.2 as the minimum supported version. Also disable commonName check and enforce
    hostname altName verification.
    """
    ctx = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)

    ctx.load_verify_locations(cadata=generate_ca_bundle())
    ctx.minimum_version = ssl.TLSVersion.TLSv1_2
    ctx.set_ciphers("DEFAULT")

    try:
        ctx.hostname_checks_common_name = False
    except AttributeError:
        pass

    try:
        ctx.check_hostname = True
    except AttributeError:
        pass

    return ctx


__all__ = (
    "root_der_certificates",
    "root_pem_certificates",
    "generate_ca_bundle",
    "create_default_ssl_context",
)



from __future__ import annotations

import typing

def root_der_certificates() -> typing.List[bytes]:
    """
    Retrieve a list of root certificate from your operating system trust store.
    They will be DER (binary) encoded.
    """
    ...
